//
//  $Id: PostfixStatement.m,v 1.1 2007/05/02 07:26:10 rh Exp $
//
//  Copyright 2007 Rene Hexel. All rights reserved.
//

#import "PostfixStatement.h"

@implementation PostfixStatement

static NSString *expression(NSScanner *scanner);	// forward reference

/*
 * Factor = number | identifier | '(' Expression ')'
 */
static NSString *factor(NSScanner *scanner)
{
	if ([scanner isAtEnd])
		return @"";

	NSString *result;
	double number;
	
	if ([scanner scanDouble: &number])
		return [NSString stringWithFormat: @"%lg", number];
	else if ([scanner scanCharactersFromSet:
					[NSCharacterSet letterCharacterSet]
				     intoString: &result])
		return result;
	else if ([scanner scanString: @"(" intoString: NULL])
	{
		result = expression(scanner);
		if (![scanner scanString: @")" intoString: NULL])
			[NSException raise: NSInvalidArgumentException
				    format: @"')' Expected at scan location %u",
				[scanner scanLocation]];
	}

	return result;
}


/*
 * Term = Factor { '*'|'/' Factor }
 */
static NSString *term(NSScanner *scanner)
{
	NSString *result = factor(scanner);
	
	while (![scanner isAtEnd])
	{
		NSCharacterSet *operators = [NSCharacterSet
			characterSetWithCharactersInString: @"*/"];
		NSString *operator;
		
		if (![scanner scanCharactersFromSet: operators
					 intoString: &operator])
			break;
		result = [NSString stringWithFormat: @"%@ %@ %@",
			result, factor(scanner), operator];
	}
	
	return result;
}


/*
 * Expression = Term { '+'|'-' Term }
 */
static NSString *expression(NSScanner *scanner)
{
	NSString *result = term(scanner);

	while (![scanner isAtEnd])
	{
		NSCharacterSet *operators = [NSCharacterSet
			characterSetWithCharactersInString: @"+-"];
		NSString *operator;

		if (![scanner scanCharactersFromSet: operators
					 intoString: &operator])
			break;
		result = [NSString stringWithFormat: @"%@ %@ %@",
			result, term(scanner), operator];
	}

	return result;
}

/*
 * Statement  = [identifier '=' ] Expression
 */
static NSString *statement(NSScanner *scanner)
{
	NSString *identifier;
	if (!scanner || [scanner isAtEnd])
		return @"";

	unsigned location = [scanner scanLocation];

	if ([scanner scanCharactersFromSet: [NSCharacterSet letterCharacterSet]
				intoString: &identifier] &&
	    [scanner scanString: @"=" intoString: NULL])
		return [NSString stringWithFormat: @"%@ %@ =",
			identifier, expression(scanner)];

	[scanner setScanLocation: location];
	return expression(scanner);
}


+ (NSString *) expressionFromInfixExpression: (NSString *) infixExpression
{
	return statement([NSScanner scannerWithString: infixExpression]);
}

@end
